/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "frame.h"
#include "display/symbol.h"
#include "rpg/world/map.h"
#include "game.h"

#include "ui/textbox.h"

char_t FRAME_BUFFER[FRAME_HEIGHT * FRAME_WIDTH];
uint8_t FRAME_COLOR[FRAME_HEIGHT * FRAME_WIDTH];

void frameInit() {
  memset(FRAME_BUFFER, ' ', FRAME_HEIGHT * FRAME_WIDTH);
  frameUIReset();
}

void frameUpdate() {
  // Draw buffer
  entity_t *ent;
  tile_t tile;
  uint32_t i;

  map_t *map = GAME.currentMap;
  if(map == NULL) return;

  // Draw the map area.
  i = 0;
  for(uint16_t y = 0; y < FRAME_HEIGHT - FRAME_BOTTOM_UI_HEIGHT; y++) {
    for(uint16_t x = 0; x < FRAME_WIDTH; x++) {
      if(x >= map->width || y >= map->height) {
        FRAME_COLOR[i] = COLOR_BLACK;
        FRAME_BUFFER[i++] = ' ';
        continue;
      }

      // Entity?
      ent = mapEntityGetByPosition(map, x, y);
      if(ent != NULL) {
        FRAME_COLOR[i] = symbolGetColorByEntity(ent);
        FRAME_BUFFER[i++] = symbolGetCharByEntity(ent);
        continue;
      }

      // Tile?
      tile = mapTileGetByPosition(map, x, y, 0);
      FRAME_COLOR[i] = symbolGetColorByTile(tile);
      FRAME_BUFFER[i++] = symbolGetCharByTile(tile);
    }
  }
}

void frameUIReset() {
  memset(
    FRAME_BUFFER + (FRAME_HEIGHT - FRAME_BOTTOM_UI_HEIGHT) * FRAME_WIDTH,
    ' ',
    FRAME_WIDTH * FRAME_BOTTOM_UI_HEIGHT
  );

  memset(
    FRAME_COLOR + (FRAME_HEIGHT - FRAME_BOTTOM_UI_HEIGHT) * FRAME_WIDTH,
    COLOR_WHITE,
    FRAME_WIDTH * FRAME_BOTTOM_UI_HEIGHT
  );
  
  // Draw UI top and bottom borders
  for(uint16_t x = 0; x < FRAME_WIDTH; x++) {
    char_t c = x == 0 || x == FRAME_WIDTH-1 ? '+' : '-';
    int32_t i = (FRAME_HEIGHT - 1) * FRAME_WIDTH + x;
    FRAME_BUFFER[i] = c;

    i = (FRAME_HEIGHT - FRAME_BOTTOM_UI_HEIGHT) * FRAME_WIDTH + x;
    FRAME_BUFFER[i] = c;
  }

  // Draw UI left and right borders
  for(
    uint16_t y = FRAME_HEIGHT - FRAME_BOTTOM_UI_HEIGHT + 1;
    y < FRAME_HEIGHT - 1;
    y++
  ) {
    int32_t i = y * FRAME_WIDTH;
    FRAME_BUFFER[i] = '|';

    i = y * FRAME_WIDTH + FRAME_WIDTH - 1;
    FRAME_BUFFER[i] = '|';
  }
}